@@ -121,7 +121,7 @@ use Data::Serializer;
 use Time::HiRes qw/ alarm sleep /;
 use File::Spec;
 
-our $VERSION = '0.14';
+our $VERSION = '0.15';
 
 extends 'Async::Simple::Task::Fork';
 
@@ -234,7 +234,9 @@ Makes child process and returns pid of child process to parent or 0 to child pro
 sub fork_child {
     my ( $self ) = @_;
 
-    my $tmp_dir = File::Spec->tmpdir();
+    # TEMP = C:\Users\XXXXXX~1\AppData\Local\Temp
+    my ( $env_tmp ) = ( $ENV{TEMP} =~ /^(.+)$/ ); # Untaint ENV: fallback, if File::Spec->tmpdir failed
+    my $tmp_dir = File::Spec->tmpdir() || $env_tmp;
 
     my $parent_writer_fname = File::Spec->catfile( $tmp_dir, '_pw_tmp_' . int( rand 999_999_999_999 ) );
     my $parent_reader_fname = File::Spec->catfile( $tmp_dir, '_pr_tmp_' . int( rand 999_999_999_999 ) );
